@@ -18,7 +18,6 @@
 #define __PARSE_UTILS_H__
 
 #include "kerncompat.h"
-#include "kernel-shared/ctree.h"
 
 u64 parse_size_from_string(const char *s);
 enum btrfs_csum_type parse_csum_type(const char *s);
@@ -30,6 +29,7 @@ int parse_range_strict(const char *range, u64 *start, u64 *end);
 int parse_bg_profile(const char *profile, u64 *flags);
 int parse_compress_type(const char *type);
 int parse_qgroupid(const char *str, u64 *qgroupid);
+u64 parse_qgroupid_or_path(const char *p);
 int fls64(u64 x);
 
 #endif
