@@ -4,6 +4,7 @@
 source "$TEST_TOP/common" || exit
 
 check_prereq mkfs.btrfs
+check_global_prereq udevadm
 check_dm_target_support linear
 
 setup_root_helper
@@ -24,7 +25,7 @@ loopdev=`run_check_stdout $SUDO_HELPER losetup --find --show img`
 run_check $SUDO_HELPER dmsetup create "$dmname" --table "0 1048576 linear $loopdev 0"
 
 # Setting up the device may need some time to appear
-sleep 5
+run_check $SUDO_HELPER udevadm settle
 if ! [ -b "$dmdev" ]; then
 	_not_run "dm device created but not visible in /dev/mapper"
 fi
@@ -43,6 +44,8 @@ run_check_stdout $SUDO_HELPER "$TOP/mkfs.btrfs" -f "$@" "$dmdev" |
 	grep -q 'SSD detected:.*yes' || _fail 'SSD not detected'
 run_check $SUDO_HELPER "$TOP/btrfs" inspect-internal dump-super "$dmdev"
 
+run_check $SUDO_HELPER udevadm settle
+
 # cleanup
 run_check $SUDO_HELPER dmsetup remove "$dmname"
 run_mayfail $SUDO_HELPER losetup -d "$loopdev"
